==> abasic_0_0_3_9.zip: abasic_0_0_3_9/variant.cpp <==
/*
 * aBasic
 * Copyright (C) 2007 m_inaba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "variant.h"

static char buf[MAX_STR_LEN];

VARIANT_ pos;

VARIANT_::VARIANT_(){
	VARIANT_::str_val="";
	VARIANT_::d_val=0;
	VARIANT_::flag=true;//l͒l^
}

VARIANT_::~VARIANT_(){
}

//---------------------------------------
void VARIANT_::operator=(char*obj){
	VARIANT_::str_val = obj;
	VARIANT_::d_val=atof(obj);//lɕϊ

	if(d_val==0 && *obj!='0'){
		VARIANT_::flag=false;//^ɐݒ
	}else{
		VARIANT_::flag=true;//l^ɐݒ
	}
}

void VARIANT_::operator=(double obj){
	VARIANT_::d_val   = obj;
	sprintf(buf,"%g",obj);//l𕶎ɕϊ
	VARIANT_::str_val=buf;
	VARIANT_::flag=true;//l^ɐݒ
}

void VARIANT_::operator=(VARIANT_*obj){
	VARIANT_::str_val = obj->str_val;
	VARIANT_::d_val   = obj->d_val;
	VARIANT_::flag=obj->flag;//^Rs[
}

void VARIANT_::operator=(int obj){
	VARIANT_::d_val   = (double)obj;
	sprintf(buf,"%g",(double)obj);//l𕶎ɕϊ
	VARIANT_::str_val=buf;
	VARIANT_::flag=true;//l^ɐݒ
}
//---------------------------------------
VARIANT_*VARIANT_::operator+(VARIANT_*obj){
	//VARIANT_ pos;
	pos.flag=flag && obj->flag; //^Rs[
	//l^̏
	pos.d_val   = VARIANT_::d_val+obj->d_val;
	//^̏
	if('\"'==obj->str_val[0]){//̐擪"폜
		pos.str_val = VARIANT_::str_val+(char*)(&(obj->str_val[1]));
	}else{
		pos.str_val = VARIANT_::str_val+obj->str_val;
	}
	return &pos;
}

VARIANT_*VARIANT_::operator+(char*obj){
	//VARIANT_ pos;
	pos.d_val = d_val+atof(obj);//lɕϊ
	
	//^̏
	if('\"'==*obj){//̐擪"폜
		pos.str_val = VARIANT_::str_val+(++obj);
	}else{
		pos.str_val = VARIANT_::str_val+obj;
	}
	pos.flag = false;//^ɐݒ
	return &pos;
}

VARIANT_*VARIANT_::operator+(double obj){
	//VARIANT_ pos;
	pos.d_val   = VARIANT_::d_val+obj;
	sprintf(buf,"%g",pos.d_val);//l𕶎ɕϊ
	pos.str_val = buf;
	pos.flag = true;//l^ɐݒ
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator-(VARIANT_*obj){
	//VARIANT_ pos;
	pos.d_val = VARIANT_::d_val-obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
VARIANT_*VARIANT_::operator-(double obj){
	//VARIANT_ pos;
	pos.d_val = VARIANT_::d_val-obj;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator*(VARIANT_*obj){
	//VARIANT_ pos;
	pos.d_val =VARIANT_::d_val*obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
VARIANT_*VARIANT_::operator*(double obj){
	//VARIANT_ pos;
	pos.d_val =VARIANT_::d_val*obj;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator/(VARIANT_*obj){
	//VARIANT_ pos;
	pos.d_val =VARIANT_::d_val/obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
VARIANT_*VARIANT_::operator/(double obj){
	//VARIANT_ pos;
	pos.d_val =VARIANT_::d_val/obj;
	pos.flag = true;//l^ɐݒ
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator==(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val==obj->d_val;
		return &pos;
	}else{
		pos=VARIANT_::str_val==obj->str_val;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator==(char*obj){
	pos=VARIANT_::str_val==obj;
	return &pos;
}

VARIANT_*VARIANT_::operator==(double obj){
	pos=VARIANT_::d_val==obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator<(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val<obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator<(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator<(double obj){
	pos=VARIANT_::d_val<obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator>(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val>obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator>(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator>(double obj){
	pos=VARIANT_::d_val>obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator<=(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val<=obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator<=(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator<=(double obj){
	pos=VARIANT_::d_val<=obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator>=(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val>=obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator>=(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator>=(double obj){
	pos=VARIANT_::d_val>=obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator!=(VARIANT_*obj){
	if(obj->flag && flag){
		pos=VARIANT_::d_val!=obj->d_val;
		return &pos;
	}else{
		pos=VARIANT_::str_val!=obj->str_val;
		return &pos;
	}
}

VARIANT_*VARIANT_::operator!=(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator!=(double obj){
	pos=VARIANT_::d_val!=obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator&&(VARIANT_*obj){
	pos=VARIANT_::d_val&&obj->d_val;
	return &pos;
}

VARIANT_*VARIANT_::operator&&(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator&&(double obj){
	pos=VARIANT_::d_val&&obj;
	return &pos;
}
//---------------------------------------
VARIANT_*VARIANT_::operator||(VARIANT_*obj){
	pos=VARIANT_::d_val||obj->d_val;
	return &pos;
}

VARIANT_*VARIANT_::operator||(char*){
	pos=0;
	return &pos;
}

VARIANT_*VARIANT_::operator||(double obj){
	pos=VARIANT_::d_val||obj;
	return &pos;
}


//-------------------------------------------------------------------------------

==> abasic_0_0_4_4.zip: abasic_0_0_4_4/variant.cpp <==
/*
 * aBasic
 * Copyright (C) 2007 m_inaba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "variant.h"

static char buf[MAX_STR_LEN];

VARIANT_ pos;

VARIANT_::VARIANT_(){
	VARIANT_::init();
}

VARIANT_::~VARIANT_(){
}

void VARIANT_::init(){
	*(VARIANT_::str_val)='\0';
	VARIANT_::d_val=0;
	VARIANT_::flag=1;//l͒l^
}

//---------------------------------------
void VARIANT_::operator=(char*obj){
	VARIANT_::init();
	VARIANT_::flag=1;//l^ɐݒ

	char*ptr=obj;
	int flag=-1;
	if('+'==*ptr || '-'==*ptr) ptr++;

	while(*ptr){
		if('.'==*ptr){
			flag++;
			if(0<flag){
				//Ȃ
				VARIANT_::flag=0;//^ɐݒ
				break;
			}
		}else if('0'>*ptr || '9'<*ptr){
			//Ȃ
			VARIANT_::flag=0;//^ɐݒ
			break;
		}
		ptr++;
	}

	if(VARIANT_::flag){
		VARIANT_::d_val=atof(obj);//lɕϊ
	}else{
		_strcpy(VARIANT_::str_val,obj);
	}
}

void VARIANT_::operator=(VARIANT_*obj){
	VARIANT_::init();
	VARIANT_::flag=obj->flag;//^Rs[
	if(VARIANT_::flag){
		VARIANT_::d_val   = obj->d_val;
	}else{
		_strcpy(VARIANT_::str_val,obj->str_val);
	}
}

void VARIANT_::operator=(int obj){
	VARIANT_::init();
	VARIANT_::d_val   = (double)obj;
	VARIANT_::flag=1;//l^ɐݒ
}

//---------------------------------------
VARIANT_*VARIANT_::operator+(VARIANT_*obj){
	pos.init();
	pos.flag=flag && obj->flag; //^Rs[
	if(pos.flag){
		//l{l
		pos.d_val = VARIANT_::d_val+obj->d_val;
	}else if(1==flag && 0==obj->flag){
		//l{
		if('\"'==*(obj->str_val)){//̐擪"폜
			sprintf(pos.str_val,"\"%g",VARIANT_::d_val);
			strcat(pos.str_val,&(obj->str_val[1]));
		}else{
			sprintf(pos.str_val,"\"%g",VARIANT_::d_val);
			strcat(pos.str_val,&(obj->str_val[0]));
		}
	}else if(0==flag && 1==obj->flag){
		//{l
		_strcpy(pos.str_val,VARIANT_::str_val);
		sprintf(buf,"%g",obj->d_val);
		strcat(pos.str_val,buf);
	}else{
		//{
		if('\"'==*(obj->str_val)){//̐擪"폜
			_strcpy(pos.str_val,VARIANT_::str_val);
			strcat(pos.str_val,&(obj->str_val[1]));
		}else{
			_strcpy(pos.str_val,VARIANT_::str_val);
			strcat(pos.str_val,&(obj->str_val[0]));
		}
	}
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator-(VARIANT_*obj){
	pos.init();
	pos.d_val = VARIANT_::d_val-obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator*(VARIANT_*obj){
	pos.init();
	pos.d_val =VARIANT_::d_val*obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator/(VARIANT_*obj){
	pos.init();
	pos.d_val =VARIANT_::d_val/obj->d_val;
	pos.flag = true;//l^ɐݒ
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator==(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		//ӂl
		pos=VARIANT_::d_val==obj->d_val;
		return &pos;
	}else if(0==obj->flag && 0==flag){
		//ӂ
		pos=!strcmp(VARIANT_::str_val,obj->str_val);
		return &pos;
	}else{
		//ȊO͑0Ԃ
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator<(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		pos=VARIANT_::d_val<obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator>(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		pos=VARIANT_::d_val>obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator<=(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		pos=VARIANT_::d_val<=obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator>=(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		pos=VARIANT_::d_val>=obj->d_val;
		return &pos;
	}else{
		pos=0;
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator!=(VARIANT_*obj){
	pos.init();
	if(obj->flag && flag){
		pos=VARIANT_::d_val!=obj->d_val;
		return &pos;
	}else{
		pos=strcmp(VARIANT_::str_val,obj->str_val);
		return &pos;
	}
}

//---------------------------------------
VARIANT_*VARIANT_::operator&&(VARIANT_*obj){
	pos.init();
	pos=VARIANT_::d_val&&obj->d_val;
	return &pos;
}

//---------------------------------------
VARIANT_*VARIANT_::operator||(VARIANT_*obj){
	pos.init();
	pos=VARIANT_::d_val||obj->d_val;
	return &pos;
}

//-------------------------------------------------------------------------------
